#############################################################
# This file was automatically generated on 2021-01-15.      #
#                                                           #
# Perl Bindings Version 2.1.29                              #
#                                                           #
# If you have a bugfix for this file and want to commit it, #
# please fix the bug in the generator. You can find a link  #
# to the generators git repository on tinkerforge.com       #
#############################################################

=pod

=encoding utf8

=head1 NAME

Tinkerforge::DeviceDisplayNames - Device Identifier to Device Display Names mapping

=cut

package Tinkerforge::DeviceDisplayNames;

use strict;
use warnings;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(get_device_display_name);

sub get_device_display_name
{
	my ($device_identifier) = @_;

	if($device_identifier == 11) { return 'DC Brick'; }
	elsif($device_identifier == 13) { return 'Master Brick'; }
	elsif($device_identifier == 14) { return 'Servo Brick'; }
	elsif($device_identifier == 15) { return 'Stepper Brick'; }
	elsif($device_identifier == 16) { return 'IMU Brick'; }
	elsif($device_identifier == 17) { return 'RED Brick'; }
	elsif($device_identifier == 18) { return 'IMU Brick 2.0'; }
	elsif($device_identifier == 19) { return 'Silent Stepper Brick'; }
	elsif($device_identifier == 21) { return 'Ambient Light Bricklet'; }
	elsif($device_identifier == 23) { return 'Current12 Bricklet'; }
	elsif($device_identifier == 24) { return 'Current25 Bricklet'; }
	elsif($device_identifier == 25) { return 'Distance IR Bricklet'; }
	elsif($device_identifier == 26) { return 'Dual Relay Bricklet'; }
	elsif($device_identifier == 27) { return 'Humidity Bricklet'; }
	elsif($device_identifier == 28) { return 'IO-16 Bricklet'; }
	elsif($device_identifier == 29) { return 'IO-4 Bricklet'; }
	elsif($device_identifier == 111) { return 'HAT Brick'; }
	elsif($device_identifier == 112) { return 'HAT Zero Brick'; }
	elsif($device_identifier == 210) { return 'Joystick Bricklet'; }
	elsif($device_identifier == 211) { return 'LCD 16x2 Bricklet'; }
	elsif($device_identifier == 212) { return 'LCD 20x4 Bricklet'; }
	elsif($device_identifier == 213) { return 'Linear Poti Bricklet'; }
	elsif($device_identifier == 214) { return 'Piezo Buzzer Bricklet'; }
	elsif($device_identifier == 215) { return 'Rotary Poti Bricklet'; }
	elsif($device_identifier == 216) { return 'Temperature Bricklet'; }
	elsif($device_identifier == 217) { return 'Temperature IR Bricklet'; }
	elsif($device_identifier == 218) { return 'Voltage Bricklet'; }
	elsif($device_identifier == 219) { return 'Analog In Bricklet'; }
	elsif($device_identifier == 220) { return 'Analog Out Bricklet'; }
	elsif($device_identifier == 221) { return 'Barometer Bricklet'; }
	elsif($device_identifier == 222) { return 'GPS Bricklet'; }
	elsif($device_identifier == 223) { return 'Industrial Digital In 4 Bricklet'; }
	elsif($device_identifier == 224) { return 'Industrial Digital Out 4 Bricklet'; }
	elsif($device_identifier == 225) { return 'Industrial Quad Relay Bricklet'; }
	elsif($device_identifier == 226) { return 'PTC Bricklet'; }
	elsif($device_identifier == 227) { return 'Voltage/Current Bricklet'; }
	elsif($device_identifier == 228) { return 'Industrial Dual 0-20mA Bricklet'; }
	elsif($device_identifier == 229) { return 'Distance US Bricklet'; }
	elsif($device_identifier == 230) { return 'Dual Button Bricklet'; }
	elsif($device_identifier == 231) { return 'LED Strip Bricklet'; }
	elsif($device_identifier == 232) { return 'Moisture Bricklet'; }
	elsif($device_identifier == 233) { return 'Motion Detector Bricklet'; }
	elsif($device_identifier == 234) { return 'Multi Touch Bricklet'; }
	elsif($device_identifier == 235) { return 'Remote Switch Bricklet'; }
	elsif($device_identifier == 236) { return 'Rotary Encoder Bricklet'; }
	elsif($device_identifier == 237) { return 'Segment Display 4x7 Bricklet'; }
	elsif($device_identifier == 238) { return 'Sound Intensity Bricklet'; }
	elsif($device_identifier == 239) { return 'Tilt Bricklet'; }
	elsif($device_identifier == 240) { return 'Hall Effect Bricklet'; }
	elsif($device_identifier == 241) { return 'Line Bricklet'; }
	elsif($device_identifier == 242) { return 'Piezo Speaker Bricklet'; }
	elsif($device_identifier == 243) { return 'Color Bricklet'; }
	elsif($device_identifier == 244) { return 'Solid State Relay Bricklet'; }
	elsif($device_identifier == 246) { return 'NFC/RFID Bricklet'; }
	elsif($device_identifier == 249) { return 'Industrial Dual Analog In Bricklet'; }
	elsif($device_identifier == 250) { return 'Accelerometer Bricklet'; }
	elsif($device_identifier == 251) { return 'Analog In Bricklet 2.0'; }
	elsif($device_identifier == 253) { return 'Load Cell Bricklet'; }
	elsif($device_identifier == 254) { return 'RS232 Bricklet'; }
	elsif($device_identifier == 255) { return 'Laser Range Finder Bricklet'; }
	elsif($device_identifier == 256) { return 'Analog Out Bricklet 2.0'; }
	elsif($device_identifier == 258) { return 'Industrial Analog Out Bricklet'; }
	elsif($device_identifier == 259) { return 'Ambient Light Bricklet 2.0'; }
	elsif($device_identifier == 260) { return 'Dust Detector Bricklet'; }
	elsif($device_identifier == 262) { return 'CO2 Bricklet'; }
	elsif($device_identifier == 263) { return 'OLED 128x64 Bricklet'; }
	elsif($device_identifier == 264) { return 'OLED 64x48 Bricklet'; }
	elsif($device_identifier == 265) { return 'UV Light Bricklet'; }
	elsif($device_identifier == 266) { return 'Thermocouple Bricklet'; }
	elsif($device_identifier == 267) { return 'Motorized Linear Poti Bricklet'; }
	elsif($device_identifier == 268) { return 'Real-Time Clock Bricklet'; }
	elsif($device_identifier == 270) { return 'CAN Bricklet'; }
	elsif($device_identifier == 271) { return 'RGB LED Bricklet'; }
	elsif($device_identifier == 272) { return 'RGB LED Matrix Bricklet'; }
	elsif($device_identifier == 276) { return 'GPS Bricklet 2.0'; }
	elsif($device_identifier == 277) { return 'RS485 Bricklet'; }
	elsif($device_identifier == 278) { return 'Thermal Imaging Bricklet'; }
	elsif($device_identifier == 279) { return 'XMC1400 Breakout Bricklet'; }
	elsif($device_identifier == 282) { return 'RGB LED Button Bricklet'; }
	elsif($device_identifier == 283) { return 'Humidity Bricklet 2.0'; }
	elsif($device_identifier == 284) { return 'Industrial Dual Relay Bricklet'; }
	elsif($device_identifier == 285) { return 'DMX Bricklet'; }
	elsif($device_identifier == 286) { return 'NFC Bricklet'; }
	elsif($device_identifier == 288) { return 'Outdoor Weather Bricklet'; }
	elsif($device_identifier == 289) { return 'Remote Switch Bricklet 2.0'; }
	elsif($device_identifier == 290) { return 'Sound Pressure Level Bricklet'; }
	elsif($device_identifier == 291) { return 'Temperature IR Bricklet 2.0'; }
	elsif($device_identifier == 292) { return 'Motion Detector Bricklet 2.0'; }
	elsif($device_identifier == 293) { return 'Industrial Counter Bricklet'; }
	elsif($device_identifier == 294) { return 'Rotary Encoder Bricklet 2.0'; }
	elsif($device_identifier == 295) { return 'Analog In Bricklet 3.0'; }
	elsif($device_identifier == 296) { return 'Solid State Relay Bricklet 2.0'; }
	elsif($device_identifier == 297) { return 'Air Quality Bricklet'; }
	elsif($device_identifier == 298) { return 'LCD 128x64 Bricklet'; }
	elsif($device_identifier == 299) { return 'Distance US Bricklet 2.0'; }
	elsif($device_identifier == 2100) { return 'Industrial Digital In 4 Bricklet 2.0'; }
	elsif($device_identifier == 2101) { return 'PTC Bricklet 2.0'; }
	elsif($device_identifier == 2102) { return 'Industrial Quad Relay Bricklet 2.0'; }
	elsif($device_identifier == 2103) { return 'LED Strip Bricklet 2.0'; }
	elsif($device_identifier == 2104) { return 'Load Cell Bricklet 2.0'; }
	elsif($device_identifier == 2105) { return 'Voltage/Current Bricklet 2.0'; }
	elsif($device_identifier == 2106) { return 'Real-Time Clock Bricklet 2.0'; }
	elsif($device_identifier == 2107) { return 'CAN Bricklet 2.0'; }
	elsif($device_identifier == 2108) { return 'RS232 Bricklet 2.0'; }
	elsif($device_identifier == 2109) { return 'Thermocouple Bricklet 2.0'; }
	elsif($device_identifier == 2110) { return 'Particulate Matter Bricklet'; }
	elsif($device_identifier == 2111) { return 'IO-4 Bricklet 2.0'; }
	elsif($device_identifier == 2112) { return 'OLED 128x64 Bricklet 2.0'; }
	elsif($device_identifier == 2113) { return 'Temperature Bricklet 2.0'; }
	elsif($device_identifier == 2114) { return 'IO-16 Bricklet 2.0'; }
	elsif($device_identifier == 2115) { return 'Analog Out Bricklet 3.0'; }
	elsif($device_identifier == 2116) { return 'Industrial Analog Out Bricklet 2.0'; }
	elsif($device_identifier == 2117) { return 'Barometer Bricklet 2.0'; }
	elsif($device_identifier == 2118) { return 'UV Light Bricklet 2.0'; }
	elsif($device_identifier == 2119) { return 'Dual Button Bricklet 2.0'; }
	elsif($device_identifier == 2120) { return 'Industrial Dual 0-20mA Bricklet 2.0'; }
	elsif($device_identifier == 2121) { return 'Industrial Dual Analog In Bricklet 2.0'; }
	elsif($device_identifier == 2122) { return 'Isolator Bricklet'; }
	elsif($device_identifier == 2123) { return 'One Wire Bricklet'; }
	elsif($device_identifier == 2124) { return 'Industrial Digital Out 4 Bricklet 2.0'; }
	elsif($device_identifier == 2125) { return 'Distance IR Bricklet 2.0'; }
	elsif($device_identifier == 2127) { return 'RGB LED Bricklet 2.0'; }
	elsif($device_identifier == 2128) { return 'Color Bricklet 2.0'; }
	elsif($device_identifier == 2129) { return 'Multi Touch Bricklet 2.0'; }
	elsif($device_identifier == 2130) { return 'Accelerometer Bricklet 2.0'; }
	elsif($device_identifier == 2131) { return 'Ambient Light Bricklet 3.0'; }
	elsif($device_identifier == 2132) { return 'Hall Effect Bricklet 2.0'; }
	elsif($device_identifier == 2137) { return 'Segment Display 4x7 Bricklet 2.0'; }
	elsif($device_identifier == 2138) { return 'Joystick Bricklet 2.0'; }
	elsif($device_identifier == 2139) { return 'Linear Poti Bricklet 2.0'; }
	elsif($device_identifier == 2140) { return 'Rotary Poti Bricklet 2.0'; }
	elsif($device_identifier == 2144) { return 'Laser Range Finder Bricklet 2.0'; }
	elsif($device_identifier == 2145) { return 'Piezo Speaker Bricklet 2.0'; }
	elsif($device_identifier == 2146) { return 'E-Paper 296x128 Bricklet'; }
	elsif($device_identifier == 2147) { return 'CO2 Bricklet 2.0'; }
	elsif($device_identifier == 2152) { return 'Energy Monitor Bricklet'; }
	elsif($device_identifier == 2153) { return 'Compass Bricklet'; }
	elsif($device_identifier == 2156) { return 'Performance DC Bricklet'; }
	elsif($device_identifier == 2157) { return 'Servo Bricklet 2.0'; }
	elsif($device_identifier == 2161) { return 'IMU Bricklet 3.0'; }
	elsif($device_identifier == 2162) { return 'Industrial Dual AC Relay Bricklet'; }

	return "Unknown Device [$device_identifier]";
}

1;
